/* 
 * @brief  RF Baseband & Interrupt Priority Test Program for uSesame
 * @param  -
 * @return -
 * 
 */
#include <stdint.h>
#include "uSesame.h"

//************** Defination *******************
#define BB_Signal                   // Output RF baseband signal
#define RF433

#define CORRECT_FRAME_SIZE  24

//************** Global variables *******************
uint8_t rxbuffer[10];                     //RF buffer 

//************** Prototype *******************
 uint8_t XTCLK_Init ( void );
 void RC10MCLK_Init ( void );
 void Read_Buffer (void);
 void RF_Init (void);

//************** Functions *******************
 /*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}

void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	                //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );                     //Selects crystal as clock source
}

void Read_Buffer ( void )
{
  int i;

  for ( i = 0; i < 10; i++)                             //Load msg. into rxbuffer moving byte9 <--> byte0, byte8 <--> byte1, etc.
  {
    rxbuffer[i] = RXDATA->BYTE[9-i]; 
  }
}

void RF_Init ( void )
{
  // RF Setup for 433.92MHz reception
  
  RF_SetState( RFEN );                  //RF Enabled
  RF_ReinitDecoder();                   	// Clear the MSGRDY bit, restarting reception
  RF_SnifferMode( SNIFDIS );             //Sniffer disbled
#ifndef BB_Signal
  RF_AgcControl( AGCEN, 0x1A );         //AGC Enabled
#else
  RF_AgcControl( AGCDIS, 0x60 );         //AGC Disabled
#endif  
  RF_SetPllBiasTime( PLL48 );           //PLL waits 48 clock cycles 
  RF_SetSleepTime( RFSLEEP10 );         //Sleep time for sniffer is 10*1024 cycles
  RF_SetWakeTime( WAKET12 );            //Wake time from sniff is 12*1024 cycles (220ms for 3.58MHz crystal) 
#ifdef RF433
  *RFNX = 0x79;                         //PLL Integer divider  0x79
  *RFNF = 0x00;                         //PLL Fractional Divider
//  *RFNF = 53;                         //PLL Fractional Divider
#else
  *RFNX = 88;                         //PLL Integer divider  315MHz
  *RFNF = 44;
#endif
#ifdef RF433
  // Hi side rejection enabled, fractional disabled, charge pump and loop filter
  RF_Setup0( HISDEN, FDIS, CPT01, 0x0A ); 
  //16 +DIV_LO2[1:0], Base band gain 00 = 0dB, LNA drain res. 1Kohm, LNA bias 1mA
  RF_Setup1( DIV_LO2_18, BBGAIN0dB, LNADR1K, LNABIAS1mA);
  //Delay timer of the slicer, Attack time of the slicer, Time to allow slicer 
  //to fast bias, Symbol Decimation rate
  RF_SlicerControl( ALPHA02, BETA02 , FT860uS , SDR03 );
#else
  // Low side rejection enabled, fractional disabled, charge pump and loop filter
  RF_Setup0( HISDDIS, FEN, CPT01, 0x0A ); 
  //16 +DIV_LO2[1:0], Base band gain 00 = 0dB, LNA drain res. 1Kohm, LNA bias 500uA
  RF_Setup1( DIV_LO2_16, BBGAIN0dB, LNADR1K, LNABIAS500uA);
  //Delay timer of the slicer, Attack time of the slicer, Time to allow slicer 
  //to fast bias, Symbol Decimation rate
  RF_SlicerControl( ALPHA03, BETA03 , FT860uS , SDR03 );
#endif  
  RF_SetMinTe( 2);                     //(1+Min_Te)*(12*16)/3.58e6 
  RF_SetMaxTe(0x1E);  //( 0x1E);                  //(1 + Min_Te + Max_Te)*(12*16)/3.58e6
  RF_SetMinGb( 0x14);                  //(1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6
//
  RF_SetSnifRt( RTPLL, RFONDIS, 0x0B );	//PLL_EN (high if analog blocks enabled), RF always on Disabled, Number of edges 
					//= 11 (to check in each sniff cycle before committing to a long wake cycle.  
					//When in sniff mode, the first SNIFF_NE edges are tested for valid timing.  
					//If any one of these first edges are badly timed, then the receiver will go to sleep.
#ifndef BB_Signal
  RF_SetMinBitNumber( CORRECT_FRAME_SIZE);//0x28 );		//Minimum number of bits = 40
#else
  RF_SetMinBitNumber( 127);
#endif
}

//************** Main program *******************
int main()
{
  if (XTCLK_Init()==FALSE)				// Initialize system clock to be the Crystal
    while (1);                                          // Signal clock fail here by blinking a LED
  CLK_RcControl( RCOFF );             	                // Disable RC 10MHz
  
  RF_Init ();				                //Intialize RF to operate receiving
#ifndef BB_Signal
  NVIC_ClearPendingIRQ(RFMSG_IRQn);                       // To clear previous INT pending
  NVIC_EnableIRQ (RFMSG_IRQn);         	                // Enable RF reception interrupt
 #endif
 
  Port_Config(PTA, 0, 0, 0x01, 0x00, 0x00, 0x00, 0x00); // PA.0 used to signal timer interrupt
  Port_Config(PTC, 0, 0, 0x03, 0x00, 0x00, 0x00, 0x00); // PC.1 used to signal RF baseband edge interrupt

//  NVIC_SetPriority(TIMER0_IRQn,3);                    // The lowest priority
  TMR_Config(0,TIMERON, -307);                          // Initialize timers to 100us
//  TMR_Config(1,TIMERON, -20);
//  TMR_Config(2,TIMERON, -30);
  
  NVIC_EnableIRQ(TIMER0_IRQn);                          // Enable the interrupts
//  NVIC_EnableIRQ(TIMER1_IRQn);
//  NVIC_EnableIRQ(TIMER2_IRQn);  

  NVIC_SetPriority(RFRE_IRQn,3);                        // The lowest priority
  NVIC_SetPriority(RFFE_IRQn,3);                        // The lowest priority
  
#ifdef BB_Signal
  NVIC_ClearPendingIRQ(RFRE_IRQn);                        // Clear the pending interrupt before enabling the interrupt
  NVIC_EnableIRQ(RFRE_IRQn);
  NVIC_ClearPendingIRQ(RFFE_IRQn);                        // Clear the pending interrupt before enabling the interrupt
  NVIC_EnableIRQ(RFFE_IRQn);
#endif

  while(1);
  
}//main()


extern void NMI_Handler( void )
{
  
}

void SysTick_Handler( void )
{
  
}

void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )      		        // IRQ 1 clock Monitor
{
  
}

void SS_Handler ( void )          		        // IRQ 2 Short Circuit
{
  
}

void PIN_Handler ( void )
{

}

void RFRE_Handler ( void )                              // RF Rising Edge
{
#ifdef BB_Signal
  *PORTC |= 0x02;
#endif
}

void RFFE_Handler ( void )                              // RF Falling Edge
{
#ifdef BB_Signal
  *PORTC &= (~0x02);
#endif
}

void RFMSG_Handler ( void )  		                // IRQ C RF Message Arrived
{
  
}

void UART_Handler( void )  		                // IRQ 8 UART
{
  
}

void LIN_Handler( void )  		                // IRQ 9 LIN
{
  
}

void SPI_Handler( void )  		                // IRQ A SPI
{
  
}

void Timer0_Handler( void )
{
  *TMR0REG = -307;   			                //Reload timer to 100us  
  *PORTA ^= 0x01;               	                //Toggle PA.0
}

void Timer1_Handler( void )
{
  *TMR1REG = -20;   		                        //Reload Register  
  *PORTC ^= 0x02;                                       //Toggle PC.1  
}

void Timer2_Handler( void )
{
  *TMR2REG = -30;   		                        //Reload Register
  *PORTC ^= 0x04;                                       //Toggle PC.2  
}

void Watchdog_Handler( void )
{
 
}

void Default_IRQ_Handler ( void )
{
  
}

void I2C_Collision_Handler( void )
{

}

void I2C_Handler( void )
{

}


//***************************** End of File ************************************