/** 
 * @brief  RF Test Program for uSesame
 * @param  -
 * @return -
 * 
 */

#include <stdint.h>
#include "usesame.h"

//************** Defination *******************
#define CORRECT_FRAME_SIZE  64


//************** Global variables *******************
uint8_t rxbuffer[10];                     //RF buffer 

//************** Prototype *******************

uint8_t XTCLK_Init ( void );
void RC10MCLK_Init ( void );
void Read_Buffer (void);
void RF_Init (void);

//************** Functions *******************

//************** Functions *******************

/*
* @param 	RC10KHZ - Clock source = RC AUX 10KHz
* @param 	RC10MHz - Clock source = RC 10MHz
* @param 	XTCLOCK - Clock source = XT       */

uint8_t XTCLK_Init ( void )
{
  int i,j;
  
  // Initialize the Crystal Clock to run the system, divided by one (default)
  j=0;
  if (!(*PMUCLK & (XTCLOCK|RC10MHZ))) CLK_SelectClockSource( RC10MHZ ); // Switch 10K to 10MHz
  while ((*PMUCLK & (XTCLOCK|RC10MHZ)) != XTCLOCK )  		//Wait for XT ON amd stable
  {
    if (!(*PMUCLK & XTON))						// XT was not enabled
    {
		CLK_CrystalControl( XTON );           	// Enable crystal oscillator
        for ( i = 0; i < 30000; i++);			// Delay
        CLK_SelectClockSource( XTCLOCK );		// Switch Crystal to be clock source
    }
    else
    {
        CLK_SelectClockSource( XTCLOCK );       // Switch Crystal to be clock source
        for ( i = 0; i < 30000; i++);			// Delay
    }
	j++;										// Error count increasd
    if (j >= 10) return (FALSE);				// Fail at Crystal init
  }
  return (TRUE);								// Successful at Crystal init
}
  
void RC10MCLK_Init ( void )
{
  // Initialize the RC10M Clock to run the system, divided by one (default)
  CLK_RcControl( RCON );             	  //Enable crystal oscillator
  CLK_SelectClockSource( RC10MHZ );       //Selects crystal as clock source
}


void Read_Buffer ( void )
{
  int i;

  for ( i = 0; i < 10; i++)           //Load msg. into rxbuffer moving byte9 <--> byte0, byte8 <--> byte1, etc.
  {
    rxbuffer[i] = RXDATA->BYTE[9-i]; 
  }
}

void RF_Init ( void )
{
  // RF Setup for 433.92MHz reception
  
  RF_SetState( RFEN );                  //RF Enabled
  RF_ReinitDecoder();                   // Clear the MSGRDY bit, starting reception  
  RF_SnifferMode( SNIFDIS );            //Sniffer disable
  RF_AgcControl( AGCEN, 0x1A );         //AGC Enabled
  RF_SetPllBiasTime( PLL48 );           //PLL waits 48 clock cycles 
  RF_SetSleepTime( RFSLEEP10 );         //Sleep time for sniffer is 10*1024 cycles
  RF_SetWakeTime( WAKET12 );            //Wake time from sniff is 12*1024 cycles (220ms for 3.58MHz crystal) 
  *RFNX = 0x79;                         //PLL Integer divider
  *RFNF = 0x00;                         //PLL Fractional Divider
  // Hi side rejection enabled, fractional disabled, charge pump and loop filter
  RF_Setup0( HISDEN, FDIS, CPT01, 0x0A ); 
  //16 +DIV_LO2[1:0], Base band gain 00 = 0dB, LNA drain res. 1Kohm, LNA bias 1mA
  RF_Setup1( DIV_LO2_18, BBGAIN0dB, LNADR1K, LNABIAS1mA);  
  //Delay timer of the slicer, Attack time of the slicer, Time to allow slicer 
  //to fast bias, Symbol Decimation rate
  RF_SlicerControl( ALPHA02, BETA02 , FT860uS , SDR03 );      
  RF_SetMinTe( 2 );                     //(1+Min_Te)*(12*16)/3.58e6 
  RF_SetMaxTe( 22 );                    //(1 + Min_Te + Max_Te)*(12*16)/3.58e6
  RF_SetMinGb( 5 );                     //(1 + Min_Te + Max_Te + Min_Gb)*(12*16)/3.58e6
  RF_SetSnifRt( RTPLL, RFONDIS, 0x0B );	//PLL_EN (high if analog blocks enabled), RF always on Disabled, Number of edges 
					//= 11 (to check in each sniff cycle before committing to a long wake cycle.  
					//When in sniff mode, the first SNIFF_NE edges are tested for valid timing.  
					//If any one of these first edges are badly timed, then the receiver will go to sleep.
  RF_SetMinBitNumber( CORRECT_FRAME_SIZE );		//Minimum number of bits = CORRECT_FRAME_SIZE
  
}


//************** Main program *******************
int main()
{
  if (XTCLK_Init()==FALSE)				// Initialize system clock to be the Crystal
    while (1);                                          // Signal clock fail here by blinking a LED
  CLK_RcControl( RCOFF );             	                // Disable RC 10MHz
  
  RF_Init ();				                //Intialize RF to operate receiving 
  NVIC_ClearPendingIRQ(RFMSG_IRQn);                    // Clear RFMSG interrupt 
  NVIC_EnableIRQ (RFMSG_IRQn);         	                //Enable RF reception interrupt
  Port_Config(PTE, 0, 0, 0x01, 0x00, 0x00, 0x00, 0x00); //PC.0, 1 and 2 used to signal interrupts  
  *PORTE=0x0F;
  
 
 
 // TMR_Config(0,TIMEROFF, -6000000);    //Initialize timer 0
 //NVIC_EnableIRQ(TIMER0_IRQn);      //Enable the interrupts
  while (1)
  {
    if (XTCLK_Init()==FALSE)				// Check system clock to be the Crystal
      while (1);                                        // Signal clock fail here by a LED  
    //*PORTE=0xFF;
  }
  
}


//************** ISR *******************
extern void NMI_Handler( void )
{
  
}

void SysTick_Handler( void )
{
  
}

void BrownOut_Handler ( void )
{
  
}

void ClkMon_Handler ( void )
{
  
}

void SS_Handler ( void )
{
  
}

void RFRE_Handler ( void )              // RF Rising Edge
{
  
}

void RFFE_Handler ( void )              // RF Falling Edge
{
  
}

void UART_Handler( void )  		// IRQ 8 UART
{
  
}

void LIN_Handler( void )  		// IRQ 9 LIN
{
  
}

void SPI_Handler( void )  		// IRQ A SPI
{
  
}

void US_Handler( void )
{
  
}

volatile uint8_t rfaux;

void RFMSG_Handler( void )  			// IRQ C - RF Message Arrived
{
  rfaux = *RXNUMBER;
  
 // *PORTC ^= 0x01;                               //Toggle PC.0  
   Read_Buffer();
  if ( *RXNUMBER <= (CORRECT_FRAME_SIZE+4) )	// If received correct number of bits
  {
  
    Read_Buffer();
//    if ( ( rxbuffer[7] == 0x05 ) && ( rxbuffer[8] == 0x08 ) && ( (rxbuffer[9] & 0xf0) == 0xC0 ) )     // Keyfob ID
    if ( ( rxbuffer[7] == 0x2C ) && ( rxbuffer[8] == 0x5B ) && ( (rxbuffer[9] & 0xf0) == 0xC0 ) )     // Keyfob ID
    {
     // TMR_Config(0,TIMERON, -6000000);    //Initialize timer 0
      *PORTE ^= 0x01;                             //Toggle PC.2  
    }
  }
  else
  {
   // *PORTC ^= 0x02;                             //Toggle PC.1  
  }
  RF_SetState(RFEN);                    	// Set the RF on
  RF_ReinitDecoder();                   	// Clear the MSGRDY bit, restarting reception
}

void Default_IRQ_Handler( void )
{
  
}

void Timer0_Handler( void )
{
  
  // TMR_Config(0,TIMEROFF, -3000000);    //Initialize timer 0
 //  *PORTE = 0xFF;                                                   

}

void Timer1_Handler( void )
{
  
}

void Timer2_Handler( void )
{
  
}

void Watchdog_Handler( void )
{
  
}

void I2C_Collision_Handler( void )
{

}

void I2C_Handler( void )
{

}

void PIN_Handler ( void )
{  
}



//***************************** End of File ************************************
